#include <stdio.h>
#include <strings.h>
#include <stdint.h>

#include "ifnow.h"
#include "ifnow_now.h"
#include "ifnow_tun.h"

ifnow_now_cfg_t now_cfg;
ifnow_tun_cfg_t tun_cfg;

#define ANSI_FONT_COL_RESET     "\x1b[0m"
#define FONT_COL_CUSTOM_RED     "\e[38;2;200;0;0m"
#define FONT_COL_CUSTOM_GREEN   "\e[38;2;0;200;0m"
#define FONT_COL_CUSTOM_BLUE    "\e[38;2;0;0;200m"
#define BCKGRD_COL_CUSTOM_RED   "\e[48;2;200;0;0m"
#define BCKGRD_COL_CUSTOM_GREEN "\e[48;2;0;200;0m"
#define BCKGRD_COL_CUSTOM_BLUE  "\e[48;2;0;0;200m"

char *color_chart[] = {
  // NOW header
  BCKGRD_COL_CUSTOM_GREEN, // category version
  FONT_COL_CUSTOM_BLUE, FONT_COL_CUSTOM_BLUE, FONT_COL_CUSTOM_BLUE, // org identifier
  FONT_COL_CUSTOM_RED, FONT_COL_CUSTOM_RED, FONT_COL_CUSTOM_RED, FONT_COL_CUSTOM_RED, // random
  // Vendor Specific Header
  BCKGRD_COL_CUSTOM_GREEN, // Element ID
  BCKGRD_COL_CUSTOM_BLUE, // Length
  FONT_COL_CUSTOM_BLUE, FONT_COL_CUSTOM_BLUE, FONT_COL_CUSTOM_BLUE, // org identifier
  BCKGRD_COL_CUSTOM_BLUE, // Type
  BCKGRD_COL_CUSTOM_GREEN, // Version
  // Body
  NULL
};
uint8_t color_size = 15;

int ifnow_now_receive_callback(void *cfg, uint8_t *buf, uint32_t len) {
  ifnow_now_cfg_t *now_cfg = cfg;

  printf("NOW receive_count: %d\n", len);
  for(uint16_t i = 0; i < len; i++) {
    printf("%s%02X"ANSI_FONT_COL_RESET" ", (i < color_size ? color_chart[i] : (buf[i] == 0x11 ? BCKGRD_COL_CUSTOM_RED : "")), buf[i]);
    if(i % 16 == 15) printf("\n");
  }
  printf("\n");

  // TODO apply buf to actual structs of headers
  uint8_t bp = 57;
  int err = ifnow_tun_send(&tun_cfg, buf + bp, len - bp);
  if(err < 0) {
    perror("Failed to send over tunnel");
  }
}

int ifnow_tun_receive_callback(void *cfg, uint8_t *buf, uint32_t len) {
  printf("TUN receive_count: %d\n", len);

  for(uint32_t i = 0; i < len; i++) {
    printf("%02X ", buf[i]);
    if(i % 16 == 15) printf("\n");
  }
  printf("\n");

  int err = ifnow_now_send(&now_cfg, buf, len);
  if(err < 0) {
    perror("Failed to send over now");
  }
}

uint8_t main(int argc, char **argv) {
  // TODO handle arguments
  // * <now_if>   wifi interface to listen for ESP-Now packets on
  //              Interface should be in monitor mode
  // * <tun_if>   name of new TUN interface
  // * <ip_addr>  IP address for `now` interface

  now_cfg.if_name = argv[1];
  now_cfg.receive = &ifnow_now_receive_callback;

  tun_cfg.if_name = argv[2];
  tun_cfg.addr_s = argv[3];
  tun_cfg.receive = &ifnow_tun_receive_callback;

  ifnow_now_init(&now_cfg);
  ifnow_tun_init(&tun_cfg);

  while(1) {};

  return 0;
}
