#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <arpa/inet.h>
#include <linux/if_packet.h>
#include <net/ethernet.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include "ifnow.h"
#include "ifnow_now.h"

ifnow_packet_t packet;

int ifnow_now_send(ifnow_now_cfg_t *now_cfg, uint8_t *buf, uint16_t len) {
  struct IEEE80211_radiotap radiotap = IFNOW_RADIOTAP_DEFAULT();
  struct IEEE80211_actionframe action = IFNOW_ACTIONFRAME_DEFAULT();
  struct IEEE80211_vendorspecific vendor = IFNOW_VENDORSPECIFIC_DEFAULT();
  struct IEEE80211_wlan wlan = IFNOW_WLAN_DEFAULT();

  memcpy(&packet.radiotap, &radiotap, sizeof(struct IEEE80211_radiotap));
  memcpy(&packet.wlan.actionframe, &action, sizeof(struct IEEE80211_actionframe));

  memcpy(&packet.wlan.actionframe.content, &vendor, sizeof(struct IEEE80211_vendorspecific));
  memcpy(&packet.wlan.actionframe.content.payload, buf, len < 250 ? len : 250);
          packet.wlan.actionframe.content.length = len + 5;

  memcpy(&packet.wlan.da, buf, 6);
  memcpy(&packet.wlan.sa, buf + 6, 6);
  memcpy(&packet.wlan.bssid, buf, 6);

  printf("NOW: sending %ld bytes\n", sizeof(packet));

  for(uint16_t i = 0; i < sizeof(packet); i++) {
    printf("%02X ", ((uint8_t*)&packet)[i]);
    if(i % 16 == 15) printf("\n");
  }
  printf("\n");
  // TODO verify sendto vs send vs write
  int ret = sendto(now_cfg->fd_if_socket, &packet, sizeof(packet), 0, NULL, 0);

  return ret;
}

void *ifnow_thread_handle(void *arg) {
  ifnow_now_cfg_t *now_cfg = arg;

  printf("starting ifnow_now thread.\n");

  ssize_t receive_count = 0;
  uint8_t receive_buffer[IFNOW_MAX_RECEIVE];

  while(1) {
    receive_count = recv(now_cfg->fd_if_socket, receive_buffer, IFNOW_MAX_RECEIVE, MSG_TRUNC);

    if(receive_count > 0 && now_cfg->receive != NULL) {
      now_cfg->receive(now_cfg, receive_buffer, receive_count);
    }
  }
}

int ifnow_now_init(ifnow_now_cfg_t *now_cfg) {
  printf("Using interface: %s\n", now_cfg->if_name);

  struct sockaddr_ll if_bind_addr;
  struct ifreq ifr;

  now_cfg->fd_if_socket = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));

  if(now_cfg->fd_if_socket == -1) {
    perror("Failed to initialize IF socket.");

    return ERR_SOCKET_FAILED;
  }

  strncpy((char*)ifr.ifr_name, now_cfg->if_name, IFNAMSIZ);

  int ioctrl_err = ioctl(now_cfg->fd_if_socket, SIOCGIFINDEX, &ifr);

  if(ioctrl_err < 0) {
    perror("Failed to set io ctl on IF socket.");
    return ERR_IOCTL_FAILED;
  }

  if_bind_addr.sll_family = PF_PACKET;
  if_bind_addr.sll_protocol = htons(ETH_P_ALL);
  if_bind_addr.sll_ifindex = ifr.ifr_ifindex;

  int bind_err = bind(now_cfg->fd_if_socket, (struct sockaddr *)&if_bind_addr, sizeof(if_bind_addr));

  if(bind_err < 0) {
    perror("Failed bind raw socket.");

    return ERR_BIND_FAILED;
  }

	now_cfg->filter.len = 53;

	uint32_t MSB_dst = 0;
	uint32_t LSB_dst = 0;

	uint32_t MSB_src = 0;
	uint32_t LSB_src = 0;

	uint8_t jeq_dst = 0x30;
	uint8_t jeq_src = 0x30;

	struct sock_filter temp_code[53] = {
			{ 0x30, 0, 0, 0x00000003 },
			{ 0x64, 0, 0, 0x00000008 },
			{ 0x7, 0, 0, 0x00000000 },
			{ 0x30, 0, 0, 0x00000002 },
			{ 0x4c, 0, 0, 0x00000000 },
			{ 0x2, 0, 0, 0x00000000 },
			{ 0x7, 0, 0, 0x00000000 },
			{ 0x50, 0, 0, 0x00000000 },
			{ 0x54, 0, 0, 0x000000fc },
			{ 0x15, 0, 42, 0x000000d0 },
			{ 0x40, 0, 0, 0x00000018 },
			{ 0x15, 0, 40, 0x7f18fe34 },
			{ 0x50, 0, 0, 0x00000020 },
			{ 0x15, 0, 38, 0x000000dd },
			{ 0x40, 0, 0, 0x00000021 },
			{ 0x54, 0, 0, 0x00ffffff },
			{ 0x15, 0, 35, 0x0018fe34 },
			{ 0x50, 0, 0, 0x00000025 },
			{ 0x15, 0, 33, 0x00000004 },
			{ 0x50, 0, 0, 0x00000000 },
			{ 0x45, 31, 0, 0x00000004 },
			{ 0x45, 0, 21, 0x00000008 },
			{ 0x50, 0, 0, 0x00000001 },
			{ 0x45, 0, 4, 0x00000001 },
			{ 0x40, 0, 0, 0x00000012 },
			{ jeq_dst, 0, 26, LSB_dst },
			{ 0x48, 0, 0, 0x00000010 },
			{ jeq_dst, 4, 24, MSB_dst },
			{ 0x40, 0, 0, 0x00000006 },
			{ jeq_dst, 0, 22, LSB_dst },
			{ 0x48, 0, 0, 0x00000004 },
			{ jeq_dst, 0, 20, MSB_dst },
			{ 0x50, 0, 0, 0x00000001 },
			{ 0x45, 0, 13, 0x00000002 },
			{ 0x45, 0, 4, 0x00000001 },
			{ 0x40, 0, 0, 0x0000001a },
			{ jeq_src, 0, 15, LSB_src },
			{ 0x48, 0, 0, 0x00000018 },
			{ jeq_src, 12, 13, MSB_src },
			{ 0x40, 0, 0, 0x00000012 },
			{ jeq_src, 0, 11, LSB_src },
			{ 0x48, 0, 0, 0x00000010 },
			{ jeq_src, 8, 9, MSB_src },
			{ 0x40, 0, 0, 0x00000006 },
			{ jeq_dst, 0, 7, LSB_dst },
			{ 0x48, 0, 0, 0x00000004 },
			{ jeq_dst, 0, 5, MSB_dst },
			{ 0x40, 0, 0, 0x0000000c },
			{ jeq_src, 0, 3, LSB_src },
			{ 0x48, 0, 0, 0x0000000a },
			{ jeq_src, 0, 1, MSB_src },
			{ 0x6, 0, 0, 0x00040000 },
			{ 0x6, 0, 0, 0x00000000 }
  };

	now_cfg->filter.filter = (struct sock_filter*) malloc(sizeof(struct sock_filter)*now_cfg->filter.len);
	memcpy(now_cfg->filter.filter, temp_code, sizeof(struct sock_filter) * now_cfg->filter.len);

  int sockopt_err = setsockopt(now_cfg->fd_if_socket, SOL_SOCKET, SO_ATTACH_FILTER, &now_cfg->filter, sizeof(now_cfg->filter));
  if(sockopt_err < 0) {
    perror("Failed to set BPF socket options.");

    return ERR_FILTER_FAILED;
  }

  struct IEEE80211_radiotap radiotap = IFNOW_RADIOTAP_DEFAULT();
  struct IEEE80211_wlan wlan = IFNOW_WLAN_DEFAULT();
  struct IEEE80211_actionframe actionframe = IFNOW_ACTIONFRAME_DEFAULT();
  struct IEEE80211_vendorspecific content = IFNOW_VENDORSPECIFIC_DEFAULT();

  memcpy(&packet.radiotap, &radiotap, sizeof(struct IEEE80211_radiotap));
  memcpy(&packet.wlan, &wlan, sizeof(struct IEEE80211_wlan));
  memcpy(&packet.wlan.actionframe, &actionframe, sizeof(struct IEEE80211_actionframe));
  memcpy(&packet.wlan.actionframe.content, &content, sizeof(struct IEEE80211_vendorspecific));

  printf("radiotap.channel_freq: %d\n", packet.radiotap.channel_freq);


  pthread_create(&now_cfg->thread_id, NULL, &ifnow_thread_handle, now_cfg);
}
